@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # A modRana QML GUI module
@@ -20,16 +19,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
-import SocketServer
-import random
+import os
 import sys
 import re
-import os
-from threading import Thread
 import traceback
 import cStringIO
-import urllib
-import urllib2
 
 # PySide
 from PySide import QtCore
@@ -39,21 +33,30 @@ from PySide.QtDeclarative import *
 from PySide.QtNetwork import *
 
 # modRana imports
-from base_gui_module import GUIModule
+import math
+from modules.gui_modules.base_gui_module import GUIModule
 from datetime import datetime
 import time
-
-import BaseHTTPServer
-import SimpleHTTPServer
+from modules.gui_modules.gui_qml import drawing
+from core.fix import Fix
 
 global globe
 
+# QML paths
+BASE_QML_FOLDER = "modules/gui_modules/gui_qml"
+QML_PLATFORM_INDEPENDENT = "qml"
+QML_HARMATTAN = "qml_harmattan"
+DEFAULT_QML_FOLDER = QML_HARMATTAN
+# TODO: use the independent GUI as default
+QML_MAIN_FILENAME = "main.qml"
+
+
 def newlines2brs(text):
   """ QML uses <br> instead of \n for linebreak """
   return re.sub('\n', '<br>', text)
 
 
-class Logger:
+class Logger(object):
   def __init__(self, log=True):
     pass
     self.log = log
@@ -65,7 +68,7 @@ class Logger:
 logger = Logger(log=False)
 
 def getModule(m, d, i):
-  return(QMLGUI(m, d, i))
+  return QMLGUI(m, d, i)
 
 
 class QMLGUI(GUIModule):
@@ -93,6 +96,18 @@ class QMLGUI(GUIModule):
         self.modrana.shutdown()
 
     self.app = QApplication(sys.argv)
+
+    # register custom modRana types
+    # NOTE: custom types need to be registered AFTER
+    # QApplication is created but BEFORE QDeclarativeView
+    # is instantiated, or else horrible breakage occurs :)
+    qmlRegisterType(drawing.PieChart, 'Charts', 1, 0, 'PieChart')
+    qmlRegisterType(drawing.PieSlice, "Charts", 1, 0, "PieSlice")
+
+    # m-declarative stuff implemented in Python
+    qmlRegisterType(Screen, "mpBackend", 1, 0, "Screen")
+    qmlRegisterType(Snapshot, "mpBackend", 1, 0, "Snapshot")
+
     self.view = ModifiedQDeclarativeView(self.modrana)
     self.window = QMainWindow()
     self.window.setWindowTitle("modRana")
@@ -174,13 +189,21 @@ class QMLGUI(GUIModule):
   def startMainLoop(self):
     """start the main loop or its equivalent"""
 
-    #    print "QML start main loop"
+    #    print("QML starting main loop")
 
     if self.modrana.dmod.startInFullscreen():
       self.toggleFullscreen()
 
     # Create an URL to the QML file
-    url = QUrl('modules/gui_modules/gui_qml/qml/main.qml')
+    QMLSubfolder = DEFAULT_QML_FOLDER
+    if self.subtypeId  == "indep":
+      QMLSubfolder = QML_PLATFORM_INDEPENDENT
+    elif self.subtypeId == "harmattan":
+      QMLSubfolder = QML_HARMATTAN
+
+    print('QML GUI subtype folder: %s' % QMLSubfolder)
+
+    url = QUrl(os.path.join(BASE_QML_FOLDER, QMLSubfolder, QML_MAIN_FILENAME))
     # Set the QML file and show
     self.view.setSource(url)
     # get the root object
@@ -194,13 +217,11 @@ class QMLGUI(GUIModule):
     if self._notificationQueue:
       for item in self._notificationQueue:
         self.notify(*item)
-
-      #    print "loaded modules:"
-      #    print sys.modules.keys()
-
+      #    print("loaded modules:")
+      #    print(sys.modules.keys())
     self.app.exec_()
 
-  #    print "QML main loop started"
+  #    print("QML main loop started")
 
   def _qtWindowClosed(self, event):
     print('Qt window closing down')
@@ -224,15 +245,15 @@ class QMLGUI(GUIModule):
   def notify(self, text, msTimeout=5000, icon=""):
     """trigger a notification using the Qt Quick Components
     InfoBanner notification"""
-    return
 
-    # QML uses <br> instead of \n for linebreak
-    text = newlines2brs(text)
-    print("QML GUI notify:\n message: %s, timeout: %d" % (text, msTimeout))
-    if self.rootObject:
-      self.rootObject.notify(text, msTimeout)
-    else:
-      self._notificationQueue.append((text, msTimeout, icon))
+#    # QML uses <br> instead of \n for linebreak
+#    text = newlines2brs(text)
+#    print("QML GUI notify:\n message: %s, timeout: %d" % (text, msTimeout))
+#    if self.rootObject:
+#      self.rootObject.notify(text, msTimeout)
+#    else:
+#      self._notificationQueue.append((text, msTimeout, icon))
+    return
 
   def openUrl(self, url):
     QDesktopServices.openUrl(url)
@@ -373,19 +394,18 @@ class IconImageProvider(QDeclarativeImageProvider):
     try:
       #TODO: theme name caching ?
       f = open('themes/%s' % iconPath, 'r')
-      #      print "ICON"
-      #      print iconPath
-      #      print size
-      #      print requestedSize
+      #      print("ICON")
+      #      print(iconPath)
+      #      print(size)
+      #      print(requestedSize)
       img = QImage()
       img.loadFromData(f.read())
       f.close()
       return img
     except Exception, e:
-      print("QML GUI: icon image provider: loading icon failed", e)
-      print iconPath
-      print 'themes/%s' % (iconPath)
-
+      print("QML GUI: icon image provider: loading icon failed")
+      print(e)
+      print(os.path.join('themes', iconPath))
 
 class TileImageProvider(QDeclarativeImageProvider):
   """
@@ -407,8 +427,8 @@ class TileImageProvider(QDeclarativeImageProvider):
     the tile info should look like this:
     layerID/zl/x/y
     """
-    #    print "IMAGE REQUESTED"
-    #    print tileInfo
+    #    print("IMAGE REQUESTED")
+    #    print(tileInfo)
     try:
       # split the string provided by QML
       split = tileInfo.split("/")
@@ -430,12 +450,13 @@ class TileImageProvider(QDeclarativeImageProvider):
       img.loadFromData(f.read())
       # cleanup
       f.close()
-      #print "OK"
+      #print("OK")
       return img
 
     except Exception, e:
-      print("QML GUI: icon image provider: loading tile failed", e)
-      print tileInfo
+      print("QML GUI: icon image provider: loading tile failed")
+      print(e)
+      print(tileInfo)
       traceback.print_exc(file=sys.stdout)
 
 
@@ -459,55 +480,19 @@ class MapTiles(QtCore.QObject):
     True - tile already in storage or in memory
     False - tile download in progress, retry in a while
     """
-    #    print layer, z, x, y
+    #    print(layer, z, x, y)
     if self.gui._mapTiles.tileInMemory(layer, z, x, y):
-    #      print "available in memory"
+    #      print("available in memory")
       return True
     elif self.gui._mapTiles.tileInStorage(layer, z, x, y):
-    #      print "available in storage"
+    #      print("available in storage")
       return True
     else: # not in memory or storage
       # add a tile download request
       self.gui._mapTiles.addTileDownloadRequest(layer, z, x, y)
-      #      print "dling try later"
+      #      print("downloading, try later")
       return False
 
-# from AGTL
-class Fix():
-  BEARING_HOLD_EPD = 90 # arbitrary, yet non-random value
-  last_bearing = 0
-  # tracking the minimum difference between a received fix time and
-  # our current internal time.
-  min_timediff = datetime.utcnow() - datetime.utcfromtimestamp(0)
-
-  def __init__(self,
-               position=None,
-               altitude=None,
-               bearing=None,
-               speed=None,
-               sats=0,
-               sats_known=0,
-               dgps=False,
-               quality=0,
-               error=0,
-               error_bearing=0,
-               timestamp=None):
-    self.position = position
-    self.altitude = altitude
-    self.bearing = bearing
-    self.speed = speed
-    self.sats = sats
-    self.sats_known = sats_known
-    self.dgps = dgps
-    self.quality = quality
-    self.error = error
-    self.error_bearing = error_bearing
-    if timestamp == None:
-      self.timestamp = datetime.utcnow()
-    else:
-      self.timestamp = timestamp
-
-
 class FixWrapper(QtCore.QObject):
   def __init__(self, fix):
     QtCore.QObject.__init__(self)
@@ -520,49 +505,113 @@ class FixWrapper(QtCore.QObject):
     logger.debug("Fix updated with data from %r" % fix)
     self.changed.emit()
 
+  def _mode(self):
+    """GPS fix mode:
+    0 - no fix
+    2 - 2D fix
+    3 - 3D fix
+    """
+    return self.data.mode
+
   def _lat(self):
-    if self.data.position != None:
+    if self.data.position is not None:
       return self.data.position[0]
     else:
       return -1
 
   def _lon(self):
-    if self.data.position != None:
+    if self.data.position is not None:
       return self.data.position[1]
     else:
       return -1
 
   def _altitude(self):
-    return self.data.altitude if self.data.altitude != None else 0
-
-  def _speed(self):
-    return self.data.speed if self.data.speed != None else 0
+    return self.data.altitude if self.data.altitude is not None else 0
 
   def _bearing(self):
-    return self.data.bearing if self.data.bearing != None else 0
+    return self.data.bearing if self.data.bearing is not None else 0
+
+  def _speed(self):
+    return self.data.speed if self.data.speed is not None else 0
+
+  def _climb(self):
+    return self.data.climb if self.data.climb is not None else -1
+
+  def _magnetic_variation(self):
+    return self.data.magnetic_variation if self.data.magnetic_variation is not None else -1
+
+  def _sats(self):
+    return self.data.sats if self.data.sats is not None else -1
+
+  def _sats_in_use(self):
+    return self.data.sats_in_use if self.data.sats_in_use is not None else -1
 
   def _error(self):
     return float(self.data.error)
 
   def _valid(self):
-    return (self.data.position != None)
+    # 0-1 = no fix, 2-3 = 2D/3D fix
+    return self.data.mode > 1
 
   def _altitude_valid(self):
-    return self.data.altitude != None
+    return self.data.altitude is not None
 
   def _speed_valid(self):
-    return self.data.speed != None
+    return self.data.speed is not None
 
+  def _bearing_valid(self):
+    return self.data.bearing is not None
+
+  def _climb_valid(self):
+    return self.data.climb is not None
+
+  def _speed_error(self):
+    return float(self.data.error)
+
+  def _horizontal_accuracy(self):
+    return self.data.horizontal_accuracy if self.data.horizontal_accuracy is not None else -1
+
+  def _vertical_accuracy(self):
+    return self.data.vertical_accuracy if self.data.vertical_accuracy is not None else -1
+
+  def _speed_accuracy(self):
+    return self.data.speed_accuracy if self.data.speed_accuracy is not None else -1
+
+  def _climb_accuracy(self):
+    climbAccuracy = self.data.climb_accuracy
+    if climbAccuracy is None or math.isnan(climbAccuracy):
+      return -1
+    else:
+      return climbAccuracy
+
+  def _time_accuracy(self):
+    return self.data.time_accuracy if self.data.time_accuracy is not None else -1
+
+  def _gps_time(self):
+    return self.data.gps_time if self.data.gps_time is not None else "not available"
+
+  mode = QtCore.Property(int, _mode, notify=changed)
   lat = QtCore.Property(float, _lat, notify=changed)
   lon = QtCore.Property(float, _lon, notify=changed)
   altitude = QtCore.Property(float, _altitude, notify=changed)
-  speed = QtCore.Property(float, _speed, notify=changed)
   bearing = QtCore.Property(float, _bearing, notify=changed)
+  speed = QtCore.Property(float, _speed, notify=changed)
+  climb = QtCore.Property(float, _climb, notify=changed)
+  magneticVariation = QtCore.Property(float, _magnetic_variation, notify=changed)
+  sats = QtCore.Property(int, _sats, notify=changed)
+  satsInUse = QtCore.Property(int, _sats_in_use, notify=changed)
   error = QtCore.Property(float, _error, notify=changed)
   valid = QtCore.Property(bool, _valid, notify=changed)
   speedValid = QtCore.Property(bool, _speed_valid, notify=changed)
+  bearingValid = QtCore.Property(bool, _bearing_valid, notify=changed)
   altitudeValid = QtCore.Property(bool, _altitude_valid, notify=changed)
-
+  climbValid = QtCore.Property(bool, _climb_valid, notify=changed)
+  horizontalAccuracy = QtCore.Property(float, _horizontal_accuracy, notify=changed)
+  verticalAccuracy = QtCore.Property(float, _vertical_accuracy, notify=changed)
+  speedAccuracy = QtCore.Property(float, _speed_accuracy, notify=changed)
+  climbAccuracy = QtCore.Property(float, _climb_accuracy, notify=changed)
+  timeAccuracy = QtCore.Property(float, _time_accuracy, notify=changed)
+  gpsTime = QtCore.Property(str, _gps_time, notify=changed)
 
 class GPSDataWrapper(QtCore.QObject):
   changed = QtCore.Signal()
@@ -586,23 +635,30 @@ class GPSDataWrapper(QtCore.QObject):
     self.gps_data = FixWrapper(fix)
     self.gps_last_good_fix = FixWrapper(fix)
     self.gps_has_fix = False
-    self.gps_status = ''
+    self.gps_mode = ''
     #self.astral = Astral()
 
-  @QtCore.Slot(bool, float, float, bool, float, bool, float, float, QtCore.QObject)
-  def positionChanged(self, valid, lat, lon, altvalid, alt, speedvalid, speed, error, timestamp):
-    if valid:
-      pos = (lat, lon)
-      self._on_good_fix(Fix(pos, alt, bearing, speed))
+#  @QtCore.Slot(bool, float, float, bool, float, bool, float, float, QtCore.QObject)
+#  def positionChanged(self, valid, lat, lon, altvalid, alt, speedvalid, speed, error, timestamp):
+#    if valid:
+#      pos = (lat, lon)
+#      self._on_good_fix(Fix(pos, alt, bearing, speed))
 
   def _posChangedCB(self, key, oldValue, newValue):
     """position changed callback"""
 
     # check validity
-    pos = self.modrana.get('pos', None)
-    if pos:
-      if self.gui._location:
-        self._on_good_fix(self.gui._location.getFix())
+    fix = None
+    try:
+      fix = self.gui._location.getFix()
+    except Exception, e:
+      print("gui_qml: can't get fix object from location module")
+      print(e)
+    if fix:
+      if fix.mode > 1: # 2 -> 2D fix, 3 -> 3D fix, 0 & 1 -> no fix
+        self._on_good_fix(fix)
+      else:
+        self._on_no_fix()
     else:
       self._on_no_fix()
 
@@ -611,35 +667,17 @@ class GPSDataWrapper(QtCore.QObject):
     self.gps_data.update(fix)
     self.gps_last_good_fix.update(fix)
     self.gps_has_fix = True
+    self.gps_mode = fix.mode
     self.changed_distance_bearing.emit()
     self.changed.emit()
 
   def _on_no_fix(self):
-    self.gps_data.update(gps_data)
+#    self.gps_data.update(gps_data)
     self.gps_has_fix = False
-    self.gps_status = "unknown"
+    self.gps_mode = 0
     self.changed_distance_bearing.emit()
     self.changed.emit()
 
-  def _on_target_changed(self, target, distance, bearing):
-    self._target_valid = (target != None)
-    self._target = CoordinateWrapper(target) if target != None else CoordinateWrapper(geo.Coordinate(0, 0))
-    self.gps_target_distance = distance
-    self.gps_target_bearing = bearing
-    self.changed_distance_bearing.emit()
-    self.changed_target.emit()
-    logger.debug("Target is now set to %r" % target)
-
-  #    def _sun_angle_valid(self):
-  #        return self.astral.get_sun_azimuth_from_fix(self.gps_last_good_fix) != None
-  #
-
-  def _target(self):
-    return self._target
-
-  def _target_valid(self):
-    return self._target_valid
-
   def _gps_data(self):
     return self.gps_data
 
@@ -649,32 +687,14 @@ class GPSDataWrapper(QtCore.QObject):
   def _gps_has_fix(self):
     return self.gps_has_fix
 
-  def _gps_target_distance_valid(self):
-    return self.gps_target_distance != None
-
-  def _gps_target_distance(self):
-    logger.debug("Target distance is %r" % self.gps_target_distance)
-    return float(self.gps_target_distance) if self._gps_target_distance_valid()  else 0
-
-  def _gps_target_bearing(self):
-    try:
-      return float(self.gps_target_bearing)
-    except TypeError:
-      return 0
-
-  def _gps_status(self):
-    return self.gps_status
+  def _gps_mode(self):
+    return self.gps_mode
 
 
   data = QtCore.Property(QtCore.QObject, _gps_data, notify=changed)
   lastGoodFix = QtCore.Property(QtCore.QObject, _gps_last_good_fix, notify=changed)
   hasFix = QtCore.Property(bool, _gps_has_fix, notify=changed)
-  #  targetValid = QtCore.Property(bool, _target_valid, notify=changed_target)
-  #  target = QtCore.Property(QtCore.QObject, _target, notify=changed_target)
-  #  targetDistanceValid = QtCore.Property(bool, _gps_target_distance_valid, notify=changed_distance_bearing)
-  #  targetDistance = QtCore.Property(float, _gps_target_distance, notify=changed_distance_bearing)
-  #  targetBearing = QtCore.Property(float, _gps_target_bearing, notify=changed_distance_bearing)
-  status = QtCore.Property(str, _gps_status, notify=changed)
+  mode = QtCore.Property(str, _gps_mode, notify=changed)
 
 
 class Options(QtCore.QObject):
@@ -684,12 +704,12 @@ class Options(QtCore.QObject):
     QtCore.QObject.__init__(self)
     self.modrana = modrana
 
-  """ like this, the function can accept
- and return different types to and from QML
- (basically anything that matches some of the decorators)
- as per PySide developers, there should be no perfromance
- penalty for doing this and the order of the decorators
- doesn't mater"""
+  # like this, the function can accept
+  # and return different types to and from QML
+  # (basically anything that matches some of the decorators)
+  # as per PySide developers, there should be no performance
+  # penalty for doing this and the order of the decorators
+  # doesn't mater
 
   @QtCore.Slot(str, bool, result=bool)
   @QtCore.Slot(str, int, result=int)
@@ -709,4 +729,113 @@ class Options(QtCore.QObject):
     """set a keys value in modRanas persistent options dictionary"""
     print("SET")
     print(key, value)
-    return self.modrana.set(key, value)
\ No newline at end of file
+    return self.modrana.set(key, value)
+
+  # while the overloaded set function works just fine,
+  # it always stores a float, even if an integer is pasted
+  # from QML
+  # so if you want to make sure an int or a float is stored
+  # in options, use the following two functions
+
+  @QtCore.Slot(str, int)
+  def setI(self, key, value):
+    print("SET int")
+    print(key, value)
+    return self.modrana.set(key, int(value))
+
+  @QtCore.Slot(str, float)
+  def setF(self, key, value):
+    print("SET float")
+    print(key, value)
+    return self.modrana.set(key, float(value))
+
+class Item(QtCore.QObject):
+  """implement m-declarative-item"""
+  wChanged = QtCore.Signal()
+  hChanged = QtCore.Signal()
+  opacityChanged = QtCore.Signal()
+
+  def __init__(self):
+    QtCore.QObject.__init__(self)
+    self.width = 854
+    self.height = 480
+    self.opacity = 1.0
+
+  def _getWidth(self):
+    return self.width
+
+  def _setWidth(self, value):
+    self.width = value
+    self.wChanged.emit()
+
+  def _getHeight(self):
+    return self.height
+
+  def _setHeight(self, value):
+    self.height = value
+    self.hChanged.emit()
+
+  def _getOpacity(self):
+    return self.opacity
+
+  def _setOpacity(self, value):
+    self.opacity = value
+    self.opacityChanged.emit()
+
+  width = QtCore.Property(int, _getWidth, _setWidth, notify=wChanged)
+  height = QtCore.Property(int, _getHeight, _setHeight, notify=hChanged)
+  opacity = QtCore.Property(int, _getOpacity, _setOpacity, notify=hChanged)
+
+class Screen(QtCore.QObject):
+  """implement m-declarative-screen"""
+
+  def __init__(self):
+    QtCore.QObject.__init__(self)
+    self.width = 854
+    self.height = 480
+
+  def _width(self):
+    return self.width
+
+  def _height(self):
+    return self.height
+
+  wChanged= QtCore.Signal()
+  hChanged= QtCore.Signal()
+  displayWidth = QtCore.Property(int, _width, notify=wChanged)
+  displayHeight = QtCore.Property(int, _height, notify=hChanged)
+
+class Snapshot(QDeclarativeItem):
+  """implement m-declarative-snapshot"""
+  wChanged = QtCore.Signal()
+  hChanged = QtCore.Signal()
+
+  def __init__(self):
+    QDeclarativeItem.__init__(self)
+    self.snapshotWidth = 854
+    self.snapshotHeight = 480
+
+  def _getSnapshotWidth(self):
+    return self.snapshotWidth
+
+  def _setSnapshotWidth(self, value):
+    pass
+
+# hotfix for recursive loop
+  # TODO: find what's causing it
+#    self.snapshotWidth = value
+#    self.wChanged.emit()
+
+  def _getSnapshotHeight(self):
+    return self.snapshotHeight
+
+  def _setSnapshotHeight(self, value):
+    pass
+# hotfix for recursive loop
+#    self.snapshotHeight = value
+#    self.hChanged.emit()
+
+  snapshotWidth = QtCore.Property(int, _getSnapshotWidth, _setSnapshotWidth, notify=wChanged)
+  snapshotHeight = QtCore.Property(int, _getSnapshotHeight, _setSnapshotHeight, notify=hChanged)
+
+
