@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # Handles downloading of map data
@@ -19,16 +18,16 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
 from __future__ import with_statement # for python 2.5
-from base_module import ranaModule
-from tilenames import *
+from modules.base_module import RanaModule
+from core.tilenames import *
 from time import clock
 import time
 import os
-import geo
+from core import geo
 from threading import Thread
 import threading
-import urllib3
-import modrana_utils
+from modules import urllib3
+from core import utils
 
 # socket timeout
 import socket
@@ -50,14 +49,14 @@ returned by the server)"
 
 
 def getModule(m, d, i):
-  return(mapData(m, d, i))
+  return MapData(m, d, i)
 
 
-class mapData(ranaModule):
+class MapData(RanaModule):
   """Handle downloading of map data"""
 
   def __init__(self, m, d, i):
-    ranaModule.__init__(self, m, d, i)
+    RanaModule.__init__(self, m, d, i)
     self.stopThreading = True
     self.dlListLock = threading.Lock() # well, its actually a set
     self.currentDownloadList = [] # list of files and urls for the current download batch
@@ -98,11 +97,11 @@ class mapData(ranaModule):
     Get tiles that need to be downloaded and look if we don't already have some of these tiles,
     then generate a set of ('url','filename') tuples and send them to the threaded downloader
     """
-    print "Checking if there are duplicated tiles"
+    print("Checking if there are duplicated tiles")
     start = clock()
     tileFolder = self._getTileFolderPath() # where should we store the downloaded tiles
     if tileFolder is None:
-      print "mapData: tile folder path unknown or unusable"
+      print("mapData: tile folder path unknown or unusable")
       return []
     layer = self.get('layer', None) # TODO: manual layer setting
     mapLayers = self.modrana.getMapLayers() # a dictionary describing supported map layers
@@ -119,8 +118,8 @@ class mapData(ranaModule):
       if not os.path.exists(filePath): # we don't have this file
         neededTiles.append(tile)
 
-    print "Downloading %d new tiles." % len(neededTiles)
-    print "Removing already available tiles from dl took %1.2f ms" % (1000 * (clock() - start))
+    print("Downloading %d new tiles." % len(neededTiles))
+    print("Removing already available tiles from dl took %1.2f ms" % (1000 * (clock() - start)))
     return neededTiles
 
 
@@ -141,7 +140,7 @@ class mapData(ranaModule):
     """
 
     tileFolder = self._getTileFolderPath() # where should we store the downloaded tiles
-    print "tiles for the batch will be downloaded to: %s" % tileFolder
+    print("tiles for the batch will be downloaded to: %s" % tileFolder)
 
     check = self.get('checkTiles', False)
     if check:
@@ -163,7 +162,7 @@ class mapData(ranaModule):
       size = int(self.get("downloadSize", 4))
       type = self.get("downloadType")
       if type != "data":
-        print "Error: mod_mapData can't download %s" % type
+        print("Error: mod_mapData can't download %s" % type)
         return
 
       # update the info when refreshing tilecount and and no dl/size estimation is active
@@ -209,7 +208,7 @@ class mapData(ranaModule):
         midZ = maxZ
       else:
         midZ = 15
-      print "max: %d, min: %d, diff: %d, middle:%d" % (maxZ, minZ, diffZ, midZ)
+      print("max: %d, min: %d, diff: %d, middle:%d" % (maxZ, minZ, diffZ, midZ))
 
       if location == "here":
         # Find which tile we're on
@@ -217,7 +216,7 @@ class mapData(ranaModule):
         if pos is not None:
           (lat, lon) = pos
           # be advised: the xy in this case are not screen coordinates but tile coordinates
-          (x, y) = latlon2xy(lat, lon, midZ)
+          (x, y) = ll2xy(lat, lon, midZ)
           tilesAroundHere = set(self.spiral(x, y, midZ, size)) # get tiles around our position as a set
           # now get the tiles from other zoomlevels as specified
           zoomlevelExtendedTiles = self.addOtherZoomlevels(tilesAroundHere, midZ, maxZ, minZ)
@@ -245,7 +244,7 @@ class mapData(ranaModule):
         proj = self.m.get('projection', None)
         (screenCenterX, screenCenterY) = proj.screenPos(0.5, 0.5) # get pixel coordinates for the screen center
         (lat, lon) = proj.xy2ll(screenCenterX, screenCenterY) # convert to geographic coordinates
-        (x, y) = latlon2xy(lat, lon, midZ) # convert to tile coordinates
+        (x, y) = ll2xy(lat, lon, midZ) # convert to tile coordinates
         tilesAroundView = set(self.spiral(x, y, midZ, size)) # get tiles around these coordinates
         # now get the tiles from other zoomlevels as specified
         zoomlevelExtendedTiles = self.addOtherZoomlevels(tilesAroundView, midZ, maxZ, minZ)
@@ -257,21 +256,21 @@ class mapData(ranaModule):
       self.set("sizeStatus", 'unknown') # first we set the size as unknown
       neededTiles = self.currentDownloadList
       layer = self.get('layer', None)
-      print "getting size"
+      print("getting size")
       if len(neededTiles) == 0:
-        print "cant get combined size, the list is empty"
+        print("cant get combined size, the list is empty")
         return
 
       if self.sizeThread is not None:
         if self.sizeThread.finished == False:
-          print "size check already in progress"
+          print("size check already in progress")
           return
 
       self.totalSize = 0
       maxThreads = self.get('maxSizeThreads', 5)
       sizeThread = self.GetSize(self, neededTiles, layer,
         maxThreads) # the second parameter is the max number of threads TODO: tweak this
-      print  "getSize received, starting sizeThread"
+      print( "getSize received, starting sizeThread")
       sizeThread.start()
       self.sizeThread = sizeThread
 
@@ -279,14 +278,14 @@ class mapData(ranaModule):
       """get tilelist and download the tiles using threads"""
       neededTiles = self.currentDownloadList
       layer = self.get('layer', None)
-      print "starting download"
+      print("starting download")
       if len(neededTiles) == 0:
-        print "cant download an empty list"
+        print("cant download an empty list")
         return
 
       if self.getFilesThread is not None:
         if self.getFilesThread.finished == False:
-          print "download already in progress"
+          print("download already in progress")
           return
 
       maxThreads = self.get('maxDlThreads', 5)
@@ -327,7 +326,7 @@ class mapData(ranaModule):
         self.scroll -= 1
         self.set('needRedraw', True)
     elif message == "down":
-      print "down"
+      print("down")
       self.scroll += 1
       self.set('needRedraw', True)
     elif message == "reset":
@@ -372,7 +371,7 @@ class mapData(ranaModule):
 
     """start of the tile splitting code"""
     previousZoomlevelTiles = None # we will split the tiles from the previous zoomlevel
-    print "splitting down"
+    print("splitting down")
     for z in range(tilesZ, maxZ): # to max zoom (fo each z we split one zoomlevel down)
       newTilesFromSplit = set() # tiles from the splitting go there
       if previousZoomlevelTiles is None: # this is the first iteration
@@ -396,12 +395,12 @@ class mapData(ranaModule):
         newTilesFromSplit.add(leftLowerTile)
         newTilesFromSplit.add(rightLowerTile)
       extendedTiles.update(newTilesFromSplit) # add the new tiles to the main set
-      print "we are at z=%d, %d new tiles from %d" % (z, len(newTilesFromSplit), (z + 1))
+      print("we are at z=%d, %d new tiles from %d" % (z, len(newTilesFromSplit), z + 1))
       previousZoomlevelTiles = newTilesFromSplit # set the new tiles s as prev. tiles for next iteration
 
     """start of the tile coordinates rounding code"""
     previousZoomlevelTiles = None # we will the tile coordinates to get tiles for the upper level
-    print "rounding up"
+    print("rounding up")
     r = range(minZ, tilesZ) # we go from the tile-z up, e.g. a sequence progressively smaller integers
     r.reverse()
     for z in r:
@@ -420,11 +419,11 @@ class mapData(ranaModule):
         upperTile = (upperTileX, upperTileY, z)
         newTilesFromRounding.add(upperTile)
       extendedTiles.update(newTilesFromRounding) # add the new tiles to the main set
-      print "we are at z=%d, %d new tiles" % (z, len(newTilesFromRounding))
+      print("we are at z=%d, %d new tiles" % (z, len(newTilesFromRounding)))
       previousZoomlevelTiles = newTilesFromRounding # set the new tiles s as prev. tiles for next iteration
 
-      print "nr of tiles after extend: %d" % len(extendedTiles)
-    print "Extend took %1.2f ms" % (1000 * (clock() - start))
+      print("nr of tiles after extend: %d" % len(extendedTiles))
+    print("Extend took %1.2f ms" % (1000 * (clock() - start)))
 
     del tiles
 
@@ -453,7 +452,8 @@ class mapData(ranaModule):
       """create the connection pool -> to facilitate socket reuse"""
       timeout = self.callback.onlineRequestTimeout
       #headers = { 'User-Agent' : "Mozilla/5.0 (compatible; MSIE 5.5; Linux)" }
-      headers = {'User-Agent': "modRana flexible GPS navigation system (compatible; Linux)"}
+      userAgent = self.callback.modrana.configs.getUserAgent()
+      headers = {'User-Agent': userAgent}
       connPool = urllib3.connection_from_url(url, headers=headers, timeout=timeout, maxsize=self.maxThreads,
         block=False)
       return connPool
@@ -472,7 +472,7 @@ class mapData(ranaModule):
       return url
 
     def run(self):
-      print "**!! batch tile size estimation is starting !!**"
+      print("**!! batch tile size estimation is starting !!**")
       maxThreads = self.maxThreads
       shutdown = threading.Event()
       localDlListLock = threading.Lock()
@@ -484,19 +484,19 @@ class mapData(ranaModule):
         t.daemon = True
         t.start()
         threads.append(t)
-      print "Added %d URLS to check for size." % self.urlCount
+      print("Added %d URLS to check for size." % self.urlCount)
       while True:
         time.sleep(self.callback.batchInfoUpdateInterval) # this governs how often we check status of the worker threads
-        print "Batch size working...",
-        print "(threads: %i)," % (threading.activeCount() - 1, ),
+        print("Batch size working...")
+        print("(threads: %i)" % (threading.activeCount() - 1, ))
         print"pending: %d, done: %d" % (len(self.neededTiles), self.processed)
         if self.quit == True: # we were ordered to quit
-          print "***get size quiting"
+          print("***get size quiting")
           shutdown.set() # dismiss all workers
           self.finished = True
           break
         if not self.neededTiles: # we processed everything
-          print "***get size finished"
+          print("***get size finished")
           shutdown.set()
           self.finished = True
           break
@@ -505,21 +505,21 @@ class mapData(ranaModule):
       """a worker thread method for estimating tile size"""
       while 1:
         if shutdown.isSet():
-          print "file size estimation thread is shutting down"
+          print("file size estimation thread is shutting down")
           break
           # try to get some work
         with localDlListLock:
           if self.neededTiles:
             item = self.neededTiles.pop()
           else:
-            print "no more work, worker quiting"
+            print("no more work, worker quiting")
             break
             # try to retrieve size of a tile
         size = 0
         try:
           size = self.getSizeForURL(item)
         except Exception, e:
-          print "exception in a get size worker thread:\n%s" % e
+          print("exception in a get size worker thread:\n%s" % e)
 
         # increment the counter or remove an available tile in a thread safe way
         if size is None: #this signalizes that the tile is available
@@ -553,11 +553,11 @@ class mapData(ranaModule):
             request = self.connPool.urlopen('HEAD', url)
             size = int(request.getheaders()['content-length'])
       except IOError:
-        print "Could not open document: %s" % url
+        print("Could not open document: %s" % url)
         size = 0 # the url errored out, so we just say it  has zero size
       except Exception, e:
-        print "error, while checking size of a tile"
-        print e
+        print("error, while checking size of a tile")
+        print(e)
         size = 0
       return size
 
@@ -634,7 +634,7 @@ class mapData(ranaModule):
       return connPool
 
     def run(self):
-      print "**!! batch tile download is starting !!**"
+      print("**!! batch tile download is starting !!**")
       maxThreads = self.maxThreads
       shutdown = threading.Event()
       incrementLock = threading.Lock()
@@ -642,23 +642,23 @@ class mapData(ranaModule):
 
       threads = []
       for i in range(0, maxThreads): # start threads
-        """start download threads"""
+        # start download threads
         t = Thread(target=self.getTilesWorker, args=(shutdown, incrementLock))
         t.daemon = True
         t.start()
         threads.append(t)
       self.callback.notificateOnce = False
-      print "minipool initialized"
-      print "Added %d URLS to download." % self.urlCount
+      print("minipool initialized")
+      print("Added %d URLS to download." % self.urlCount)
       while True:
         time.sleep(self.callback.batchInfoUpdateInterval)
-        print "Batch tile dl working...",
-        print"(threads: %i)" % (threading.activeCount() - 1, ),
-        print"pending: %d, done: %d" % (len(self.neededTiles), self.processed)
+        print("Batch tile dl working...")
+        print("(threads: %i)" % (threading.activeCount() - 1, ))
+        print("pending: %d, done: %d" % (len(self.neededTiles), self.processed))
         # there is some downloading going on so a notification will be needed
         self.callback.notificateOnce = True
         if self.quit == True: # we were ordered to quit
-          print "***get tiles quiting"
+          print("***get tiles quiting")
           shutdown.set() # dismiss all workers
           self.finished = True
           break
@@ -677,7 +677,7 @@ class mapData(ranaModule):
             workFinished = True
 
         if workFinished: # check if we are done
-          print "***get tiles finished"
+          print("***get tiles finished")
           if self.callback.notificateOnce:
             self.callback.sendMessage('ml:notification:m:Batch download complete.;7')
             self.callback.notificateOnce = False
@@ -689,14 +689,14 @@ class mapData(ranaModule):
     def getTilesWorker(self, shutdown, incrementLock):
       while 1:
         if shutdown.isSet():
-          print "file download thread is shutting down"
+          print("file download thread is shutting down")
           break
           # try to get some work
         try:
           with self.callback.dlListLock:
             item = self.neededTiles.pop()
         except: # start the loop from beginning, so that shutdown can be checked and the thread can exit
-          print "waiting for more work"
+          print("waiting for more work")
           time.sleep(2)
           continue
 
@@ -708,7 +708,7 @@ class mapData(ranaModule):
         except Exception, e:
           failed = True
           # TODO: try to re-download failed tiles
-          print "exception in get tiles thread:\n%s" % e
+          print("exception in get tiles thread:\n%s" % e)
           #          import traceback, sys
         #          traceback.print_exc(file=sys.stdout) # find what went wrong
         # increment the counter in a thread safe way
@@ -742,7 +742,7 @@ class mapData(ranaModule):
 
           TODO: does someone supply non-bitmap/SVG tiles ?
           """
-          if modrana_utils.isTheStringAnImage(content):
+          if utils.isTheStringAnImage(content):
             #its an image, save it
             m.automaticStoreTile(content, folderPrefix, z, x, y, layerType, filename)
           else:
@@ -768,7 +768,7 @@ class mapData(ranaModule):
     """for now we are downloading just tiles,
     so I modified this to round the coordinates right after we get them"""
 
-    class spiraller:
+    class spiraller(object):
       def __init__(self, x, y, z):
         self.x = x
         self.y = y
@@ -826,7 +826,7 @@ class mapData(ranaModule):
     for point in route: #now we iterate over all points of the route
       lat, lon = point[0], point[1]
       # be advised: the xy in this case are not screen coordinates but tile coordinates
-      (x, y) = latlon2xy(lat, lon, z)
+      (x, y) = ll2xy(lat, lon, z)
       # the spiral gives us tiles around coordinates for a given radius
       currentPointTiles = self.spiral(x, y, z, radius)
       """now we take the resulting list  and process it in such a way,
@@ -834,8 +834,8 @@ class mapData(ranaModule):
       so we will save only unique tiles"""
       outputSet = set(map(lambda x: tuple(x), currentPointTiles))
       tilesToDownload.update(outputSet)
-    print "Listing tiles took %1.2f ms" % (1000 * (clock() - start))
-    print "unique tiles %d" % len(tilesToDownload)
+    print("Listing tiles took %1.2f ms" % (1000 * (clock() - start)))
+    print("unique tiles %d" % len(tilesToDownload))
     return tilesToDownload
 
   def addPointsToLine(self, lat1, lon1, lat2, lon2, maxDistance):
@@ -930,7 +930,7 @@ class mapData(ranaModule):
     if menuName == 'chooseRouteForDl':
       menus = self.m.get('menu', None)
       tracks = self.m.get('loadTracklogs', None).getTracklogList()
-      print tracks
+      print(tracks)
 
       def describeTracklog(index, category, tracks):
         """describe a tracklog list item"""
