@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # A turn by turn navigation module.
@@ -18,11 +17,13 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
+from modules.base_module import RanaModule
+from core import geo
 from threading import Thread
-from base_module import ranaModule
-import geo
 import math
 import time
+
+
 import instructions_generator
 
 REROUTE_CHECK_INTERVAL = 5000 # in ms
@@ -46,13 +47,13 @@ if gs.GUIString == "GTK":
   import pangocairo
 
 def getModule(m,d,i):
-  return turnByTurn(m,d,i)
+  return TurnByTurn(m,d,i)
 
-class turnByTurn(ranaModule):
+class TurnByTurn(RanaModule):
   """A turn by turn navigation module."""
   
   def __init__(self, m, d, i):
-    ranaModule.__init__(self, m, d, i)
+    RanaModule.__init__(self, m, d, i)
     gui = self.modrana.gui
     # initial colors
     self.navigationBoxBackground = (0,0,1,0.3) # very transparent blue
@@ -499,9 +500,9 @@ class turnByTurn(ranaModule):
           pos2nextStep = geo.distance(lat,lon,nsLat,nsLon)*1000
           pos2currentStep = geo.distance(lat,lon,csLat,csLon)*1000
           currentStep2nextStep = geo.distance(csLat,csLon,nsLat,nsLon)*1000
-#          print "pos",(lat,lon)
-#          print "cs",(csLat,csLon)
-#          print "ns",(nsLat,nsLon)
+#          print("pos",(lat,lon))
+#          print("cs",(csLat,csLon))
+#          print("ns",(nsLat,nsLon))
           print("position to next turn: %f m" % pos2nextStep)
           print("position to current turn: %f m" % pos2currentStep)
           print("current turn to next turn: %f m" % currentStep2nextStep)
@@ -754,7 +755,7 @@ class turnByTurn(ranaModule):
       return minDistance*1000 < threshold
 
   def _startTBTWorker(self):
-    print "tbt: starting worker thread"
+    print("tbt: starting worker thread")
     startThread = True
     if not self.TBTWorker: # reuse previous thread or start new one
       self.TBTWorkerEnabled = True
@@ -763,7 +764,7 @@ class turnByTurn(ranaModule):
       t.start()
       self.TBTWorker = t
     else:
-      print "tbt: reusing worker thread"
+      print("tbt: reusing worker thread")
 
   def _stopTBTWorker(self):
     self.TBTWorkerEnabled = False
@@ -804,9 +805,3 @@ class turnByTurn(ranaModule):
   def shutdown(self):
     # cleanup
     self.stopTBT()
-
-if(__name__ == "__main__"):
-  a = example({}, {})
-  a.update()
-  a.update()
-  a.update()
